
package ru.dbotthepony.mc.otm.datagen.recipes

import com.google.common.collect.ImmutableList
import net.minecraft.advancements.CriterionTriggerInstance
import net.minecraft.data.recipes.FinishedRecipe
import net.minecraft.data.recipes.RecipeBuilder
import net.minecraft.data.recipes.ShapedRecipeBuilder
import net.minecraft.data.recipes.ShapelessRecipeBuilder
import net.minecraft.resources.ResourceLocation
import net.minecraft.tags.TagKey
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.crafting.Ingredient
import net.minecraft.world.item.crafting.RecipeSerializer
import net.minecraft.world.level.ItemLike
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.core.registryName
import ru.dbotthepony.mc.otm.recipe.EnergyContainerRecipe
import java.util.function.Consumer
import java.util.stream.Stream

/**
 * because you need [equals]
 */
private interface RecipeCell {
	val value: Ingredient
}

private data class IngredientRecipeCell(override val value: Ingredient) : RecipeCell

@Suppress("EqualsOrHashCode")
private data class ItemRecipeCell(val item: ItemLike) : RecipeCell {
	override val value: Ingredient by lazy { Ingredient.of(item) }

	override fun equals(other: Any?): Boolean {
		if (other is ItemLike) {
			return other.asItem() == item
		}

		if (other is ItemRecipeCell) {
			return item == other.item
		}

		if (other is CollectionRecipeCell) {
			return other.items.size == 1 && other.items.all { it.item == item && it.tag == null }
		}

		return false
	}
}

private data class TagRecipeCell(private val item: TagKey<Item>) : RecipeCell {
	override val value: Ingredient by lazy { Ingredient.of(item) }
}

private data class ItemStackRecipeCell(private val item: ItemStack) : RecipeCell {
	override val value: Ingredient by lazy { Ingredient.of(item) }
}

@Suppress("EqualsOrHashCode")
private class CollectionRecipeCell : RecipeCell {
	val items: ImmutableList<ItemStack>

	constructor(input: Iterable<ItemStack>) : this(input.iterator())

	constructor(input: Iterator<ItemStack>) {
		val builder = ImmutableList.Builder<ItemStack>()
		for (value in input) builder.add(value)
		items = builder.build()
	}

	constructor(input: Stream<ItemStack>) {
		items = input.collect(ImmutableList.toImmutableList())
	}

	constructor(input: Array<out ItemStack>) : this(input.iterator())

	private fun items() = items
	override val value: Ingredient by lazy { Ingredient.of(items().stream()) }

	override fun equals(other: Any?): Boolean {
		if (other is CollectionRecipeCell) {
			return other.items.all { a -> items.any { b -> a.equals(b, false) } }
		}

		if (other is ItemRecipeCell) {
			return items.size == 1 && items.all { it.item == other.item && it.tag == null }
		}

		return false
	}
}

private fun makeCell(input: Ingredient?): RecipeCell? = input?.let(::IngredientRecipeCell)
private fun makeCell(input: ItemLike?): RecipeCell? = input?.let(::ItemRecipeCell)
private fun makeCell(input: TagKey<Item>?): RecipeCell? = input?.let(::TagRecipeCell)
private fun makeCell(input: Iterable<ItemStack>?): RecipeCell? = input?.let(::CollectionRecipeCell)
private fun makeCell(input: Iterator<ItemStack>?): RecipeCell? = input?.let(::CollectionRecipeCell)
private fun makeCell(input: Stream<ItemStack>?): RecipeCell? = input?.let(::CollectionRecipeCell)
private fun makeCell(input: Array<out ItemStack>?): RecipeCell? = input?.let(::CollectionRecipeCell)
private fun makeCell(input: ItemStack?): RecipeCell? = input?.let(::ItemStackRecipeCell)

private data class RecipeRow(
	val a: RecipeCell?,
	val b: RecipeCell?,
	val c: RecipeCell?,
)

/**
 * [ShapedRecipeBuilder] that doesn't suck
 */
@Suppress("unused")
class MatteryRecipe(val result: ItemLike, val count: Int = 1) {
	private val rows = arrayOfNulls<RecipeRow>(3)
	private var index = 0

	private val unlockedBy = ArrayList<Pair<String, CriterionTriggerInstance>>().also {
		it.add("has_result" to has(result))
	}

	fun unlockedBy(name: String, trigger: CriterionTriggerInstance): MatteryRecipe {
		unlockedBy.add(name to trigger)
		return this
	}

	fun unlockedBy(item: ItemLike): MatteryRecipe {
		val location = item.asItem().registryName!!
		return unlockedBy("has_${location.namespace}_${location.path}", has(item))
	}

	fun unlockedBy(item: TagKey<Item>): MatteryRecipe {
		return unlockedBy("has_${item.location.namespace}_${item.location.path}", has(item))
	}

	companion object {
		private val charlist = charArrayOf(
			'A',
			'B',
			'C',

			'D',
			'E',
			'F',

			'G',
			'K',
			'M',
		)

		private fun findOrGetNextChar(
			iterator: Iterator<Char>,
			input: RecipeCell?,
			list: MutableList<Pair<Char, RecipeCell>>,
			builder: ShapedRecipeBuilder
		): Char {
			if (input == null) {
				return ' '
			}

			val find = list.firstOrNull { it.second == input }

			if (find != null) {
				return find.first
			}

			val char = iterator.next()
			list.add(char to input)
			builder.define(char, input.value)
			return char
		}
	}

	fun build(consumer: Consumer<FinishedRecipe>, name: String? = null) {
		if (index == 0) {
			throw NoSuchElementException("No recipe rows were defined")
		}

		val builder = ShapedRecipeBuilder(result, count)

		val pairs = ArrayList<Pair<Char, RecipeCell>>()
		val iterator = charlist.iterator()

		for (row in rows) {
			row ?: break

			val a = findOrGetNextChar(iterator, row.a, pairs, builder)
			val b = findOrGetNextChar(iterator, row.b, pairs, builder)
			val c = findOrGetNextChar(iterator, row.c, pairs, builder)

			builder.pattern("$a$b$c")
		}

		for ((a, b) in unlockedBy) {
			builder.unlockedBy(a, b)
		}

		if (name != null) {
			builder.save(consumer, ResourceLocation(OverdriveThatMatters.MOD_ID,
				if (result.asItem().registryName!!.namespace == OverdriveThatMatters.MOD_ID)
					"${result.asItem().registryName!!.path}_$name"
				else
					"${result.asItem().registryName!!.namespace}_${result.asItem().registryName!!.path}_$name"
			))
		} else {
			builder.save(consumer)
		}
	}

	fun buildEnergetic(consumer: Consumer<FinishedRecipe>, name: String? = null) {
		build({
			consumer.accept(object : FinishedRecipe by it {
				override fun getType(): RecipeSerializer<*> {
					return EnergyContainerRecipe.Companion
				}
			})
		}, name)
	}

	fun row(a: Ingredient? = null, b: Ingredient? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Ingredient? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Ingredient? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Ingredient? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Ingredient? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Ingredient? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Ingredient? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Ingredient? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemLike? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemLike? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemLike? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemLike? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemLike? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemLike? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemLike? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemLike? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: TagKey<Item>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: TagKey<Item>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: TagKey<Item>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: TagKey<Item>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: TagKey<Item>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: TagKey<Item>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: TagKey<Item>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: TagKey<Item>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterable<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterable<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterable<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterable<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterable<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterable<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterable<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterable<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterator<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterator<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterator<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterator<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterator<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterator<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterator<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Iterator<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Array<out ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Array<out ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Array<out ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Array<out ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Array<out ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Array<out ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Array<out ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Array<out ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Stream<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Stream<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Stream<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Stream<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Stream<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Stream<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Stream<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: Stream<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemStack? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemStack? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemStack? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemStack? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemStack? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemStack? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemStack? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Ingredient? = null, b: ItemStack? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Ingredient? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Ingredient? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Ingredient? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Ingredient? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Ingredient? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Ingredient? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Ingredient? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Ingredient? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemLike? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemLike? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemLike? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemLike? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemLike? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemLike? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemLike? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemLike? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: TagKey<Item>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: TagKey<Item>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: TagKey<Item>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: TagKey<Item>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: TagKey<Item>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: TagKey<Item>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: TagKey<Item>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: TagKey<Item>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterable<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterable<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterable<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterable<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterable<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterable<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterable<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterable<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterator<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterator<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterator<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterator<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterator<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterator<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterator<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Iterator<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Array<out ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Array<out ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Array<out ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Array<out ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Array<out ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Array<out ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Array<out ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Array<out ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Stream<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Stream<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Stream<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Stream<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Stream<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Stream<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Stream<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: Stream<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemStack? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemStack? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemStack? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemStack? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemStack? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemStack? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemStack? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemLike? = null, b: ItemStack? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Ingredient? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Ingredient? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Ingredient? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Ingredient? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Ingredient? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Ingredient? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Ingredient? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Ingredient? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemLike? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemLike? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemLike? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemLike? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemLike? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemLike? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemLike? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemLike? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: TagKey<Item>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: TagKey<Item>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: TagKey<Item>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: TagKey<Item>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: TagKey<Item>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: TagKey<Item>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: TagKey<Item>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: TagKey<Item>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterable<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterable<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterable<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterable<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterable<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterable<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterable<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterable<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterator<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterator<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterator<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterator<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterator<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterator<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterator<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Iterator<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Array<out ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Array<out ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Array<out ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Array<out ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Array<out ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Array<out ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Array<out ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Array<out ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Stream<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Stream<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Stream<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Stream<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Stream<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Stream<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Stream<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: Stream<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemStack? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemStack? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemStack? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemStack? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemStack? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemStack? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemStack? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: TagKey<Item>? = null, b: ItemStack? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Ingredient? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Ingredient? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Ingredient? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Ingredient? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Ingredient? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Ingredient? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Ingredient? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Ingredient? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemLike? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemLike? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemLike? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemLike? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemLike? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemLike? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemLike? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemLike? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: TagKey<Item>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: TagKey<Item>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: TagKey<Item>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: TagKey<Item>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: TagKey<Item>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: TagKey<Item>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: TagKey<Item>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: TagKey<Item>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Array<out ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Array<out ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Array<out ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Array<out ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Array<out ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Array<out ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Array<out ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Array<out ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Stream<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Stream<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Stream<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Stream<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Stream<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Stream<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Stream<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: Stream<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemStack? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemStack? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemStack? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemStack? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemStack? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemStack? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemStack? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterable<ItemStack>? = null, b: ItemStack? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Ingredient? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Ingredient? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Ingredient? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Ingredient? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Ingredient? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Ingredient? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Ingredient? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Ingredient? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemLike? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemLike? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemLike? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemLike? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemLike? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemLike? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemLike? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemLike? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: TagKey<Item>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: TagKey<Item>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: TagKey<Item>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: TagKey<Item>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: TagKey<Item>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: TagKey<Item>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: TagKey<Item>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: TagKey<Item>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Array<out ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Array<out ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Array<out ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Array<out ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Array<out ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Array<out ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Array<out ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Array<out ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Stream<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Stream<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Stream<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Stream<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Stream<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Stream<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Stream<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: Stream<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemStack? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemStack? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemStack? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemStack? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemStack? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemStack? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemStack? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Iterator<ItemStack>? = null, b: ItemStack? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Ingredient? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Ingredient? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Ingredient? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Ingredient? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Ingredient? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Ingredient? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Ingredient? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Ingredient? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemLike? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemLike? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemLike? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemLike? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemLike? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemLike? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemLike? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemLike? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: TagKey<Item>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: TagKey<Item>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: TagKey<Item>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: TagKey<Item>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: TagKey<Item>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: TagKey<Item>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: TagKey<Item>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: TagKey<Item>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterable<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterable<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterable<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterator<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterator<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Iterator<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Array<out ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Array<out ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Array<out ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Array<out ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Array<out ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Array<out ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Array<out ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Array<out ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Stream<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Stream<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Stream<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Stream<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Stream<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Stream<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Stream<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: Stream<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemStack? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemStack? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemStack? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemStack? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemStack? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemStack? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemStack? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Array<out ItemStack>? = null, b: ItemStack? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Ingredient? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Ingredient? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Ingredient? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Ingredient? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Ingredient? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Ingredient? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Ingredient? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Ingredient? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemLike? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemLike? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemLike? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemLike? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemLike? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemLike? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemLike? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemLike? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: TagKey<Item>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: TagKey<Item>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: TagKey<Item>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: TagKey<Item>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: TagKey<Item>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: TagKey<Item>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: TagKey<Item>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: TagKey<Item>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterable<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Iterator<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Array<out ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Array<out ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Array<out ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Array<out ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Array<out ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Array<out ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Array<out ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Array<out ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Stream<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Stream<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Stream<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Stream<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Stream<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Stream<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Stream<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: Stream<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemStack? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemStack? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemStack? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemStack? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemStack? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemStack? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemStack? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: Stream<ItemStack>? = null, b: ItemStack? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Ingredient? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Ingredient? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Ingredient? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Ingredient? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Ingredient? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Ingredient? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Ingredient? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Ingredient? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemLike? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemLike? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemLike? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemLike? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemLike? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemLike? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemLike? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemLike? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: TagKey<Item>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: TagKey<Item>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: TagKey<Item>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: TagKey<Item>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: TagKey<Item>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: TagKey<Item>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: TagKey<Item>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: TagKey<Item>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterable<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterable<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterable<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterable<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterable<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterable<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterable<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterable<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterator<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterator<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterator<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterator<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterator<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterator<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterator<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Iterator<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Array<out ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Array<out ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Array<out ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Array<out ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Array<out ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Array<out ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Array<out ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Array<out ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Stream<ItemStack>? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Stream<ItemStack>? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Stream<ItemStack>? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Stream<ItemStack>? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Stream<ItemStack>? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Stream<ItemStack>? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Stream<ItemStack>? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: Stream<ItemStack>? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemStack? = null, c: Ingredient? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemStack? = null, c: ItemLike? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemStack? = null, c: TagKey<Item>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemStack? = null, c: Iterable<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemStack? = null, c: Iterator<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemStack? = null, c: Array<out ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemStack? = null, c: Stream<ItemStack>? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}

	fun row(a: ItemStack? = null, b: ItemStack? = null, c: ItemStack? = null): MatteryRecipe {
		check(index < 3) { "Already have all rows defined" }
		rows[index++] = RecipeRow(makeCell(a), makeCell(b), makeCell(c))
		return this
	}
}
